@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_c =
- *   "$Id: client.c,v 1.10 2002/05/31 03:44:54 mwedel Exp $";
+ *   "$Id: client.c,v 1.17 2003/07/08 12:52:35 tchize Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -50,6 +50,7 @@
 
 char *server=SERVER,*client_libdir=NULL,*meta_server=META_SERVER;
 char *image_file="";
+char *skill_names[MAX_SKILL];
 
 int meta_port=META_PORT, want_skill_exp=0,
     replyinfo_status=0, requestinfo_sent=0, replyinfo_last_face=0,
@@ -65,10 +66,6 @@ char *resists_name[NUM_RESISTS] = {
 "t undead", "fear", "depl","death", 
 "hword", "blind"};
 
-char *skill_names[MAX_SKILL] = {
-"agility", "personality", "mental", "physique", "magic", "wisdom"
-};
-
 typedef void (*CmdProc)(unsigned char *, int len);
 
 struct CmdMapping {
@@ -86,6 +83,7 @@ struct CmdMapping commands[] = {
     { "map_scroll", (CmdProc)map_scrollCmd },
     { "magicmap", MagicMapCmd},
     { "newmap", NewmapCmd },
+    { "mapextended", MapExtendedCmd },
 
     { "item1", Item1Cmd },
     { "item2", Item2Cmd },
@@ -105,6 +103,7 @@ struct CmdMapping commands[] = {
 
     { "sound", SoundCmd},
     { "anim", AnimCmd},
+    { "smooth", SmoothCmd},
 
     { "player", PlayerCmd },
     { "comc", CompleteCmd},
@@ -114,6 +113,7 @@ struct CmdMapping commands[] = {
     { "version", (CmdProc)VersionCmd },
     { "goodbye", (CmdProc)GoodbyeCmd },
     { "setup", (CmdProc)SetupCmd},
+    { "ExtendedInfoSet", (CmdProc)ExtendedInfoSetCmd},
 
     { "query", (CmdProc)handle_query},
     { "replyinfo", (CmdProc)ReplyInfoCmd},
@@ -137,7 +137,7 @@ void DoClient(ClientSocket *csocket)
 	    return;
 	}
 	if (i==0) return;   /* Don't have a full packet */
-	csocket->inbuf.buf[csocket->inbuf.len]='\0';
+	csocket->inbuf.buf[csocket->inbuf.len]='\0';    
         data = (unsigned char *)strchr((char*)csocket->inbuf.buf +2, ' ');
 	if (data) {
 	    *data='\0';
@@ -236,6 +236,8 @@ int init_connection(char *host, int port)
 
 void negotiate_connection(int sound)
 {
+    int tries;
+
     SendVersion(csocket);
 
     /* We need to get the version command fairly early on because
@@ -245,8 +247,19 @@ void negotiate_connection(int sound)
      * However, if it doesn't send the version command, we have no idea
      * what we are dealing with.
      */
+    tries=0;
     while (csocket.cs_version==0) {
 	DoClient(&csocket);
+	if (csocket.fd == -1) return;
+
+	usleep(10*1000);    /* 10 milliseconds */
+	tries++;
+	/* If we have't got a response in 10 seconds, bail out */
+	if (tries > 1000) {
+	    close(csocket.fd);
+	    csocket.fd=-1;
+	    return;
+	}
     }
 
     if (csocket.sc_version<1023) {
@@ -261,9 +274,9 @@ void negotiate_connection(int sound)
      */
     if (face_info.want_faceset) face_info.faceset = atoi(face_info.want_faceset);
     cs_print_string(csocket.fd,
-	    "setup map1acmd 1 sound %d sexp %d darkness %d newmapcmd 1 faceset %d facecache %d itemcmd 2",
+	    "setup map1acmd 1 sound %d sexp %d darkness %d newmapcmd 1 faceset %d facecache %d exp64 1 itemcmd 2",
 	    sound>=0, want_skill_exp, 
-		    want_config[CONFIG_FOGWAR], face_info.faceset,
+		    want_config[CONFIG_LIGHTING]?1:0, face_info.faceset,
 		    want_config[CONFIG_CACHE]);
 
     use_config[CONFIG_MAPHEIGHT]=want_config[CONFIG_MAPHEIGHT];
@@ -271,6 +284,11 @@ void negotiate_connection(int sound)
     if (use_config[CONFIG_MAPHEIGHT]!=11 || use_config[CONFIG_MAPWIDTH]!=11)
 	cs_print_string(csocket.fd,"setup mapsize %dx%d",use_config[CONFIG_MAPWIDTH], use_config[CONFIG_MAPHEIGHT]);
 
+    use_config[CONFIG_SMOOTH]=want_config[CONFIG_SMOOTH];
+    if (use_config[CONFIG_SMOOTH]){ /*or other mapextended infos*/
+        cs_print_string(csocket.fd,"setup extendedMapInfos 1");
+        /*will handle all special infos requested when setup answer this command*/
+    }
 
     /* If the server will answer the requestinfo for image_info and image_data,
      * send it and wait for the response.
@@ -289,6 +307,12 @@ void negotiate_connection(int sound)
 
 	do {
 	    DoClient(&csocket);
+
+	    /* it's rare, the connection can die while getting
+	     * this info.
+	     */
+	    if (csocket.fd == -1) return;
+
 	    if (use_config[CONFIG_DOWNLOAD]) {
 		/* we need to know how many faces to
 		 * be able to make the request intelligently.
@@ -323,6 +347,13 @@ void negotiate_connection(int sound)
 		    }
 		} /* Still have image_sums request to send */
 	    } /* endif download all faces */
+
+	    usleep(10*1000);    /* 10 milliseconds */
+	    /* Don't put in an upper time limit with tries like we did above - if the
+	     * player is downloading all the images, the time this takes could be
+	     * considerable.
+	     */
+
 	} while (replyinfo_status != requestinfo_sent);
     }
     if (use_config[CONFIG_DOWNLOAD]) {
