@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_client_h =
- *   "$Id: client.h,v 1.7 2002/07/05 03:25:06 mwedel Exp $";
+ *   "$Id: client.h,v 1.14 2003/10/09 06:40:20 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -54,7 +54,7 @@ char VERSION_INFO[256];
 /* How many skill types server supports/client will get sent to it.
  * If more skills are added to server, this needs to get increased.
  */
-#define MAX_SKILL   6
+#define MAX_SKILL   CS_NUM_SKILLS
 
 #define MAXANIM 2000
 
@@ -74,6 +74,15 @@ typedef struct Animations {
 
 extern Animations animations[MAXANIM];
 
+#define MAXSMOOTH 1000
+typedef struct Smooths {
+    uint8  received:1;
+    uint16 smoothid;
+    uint16  face;
+} Smooths;
+
+extern Smooths smooths[MAXSMOOTH];
+extern int smoothused;
 
 /* ClientSocket could probably hold more of the global values - it could
  * probably hold most all socket/communication related values instead
@@ -151,7 +160,11 @@ typedef enum rangetype {
 #define CONFIG_DARKNESS	    24
 #define CONFIG_PORT	    25		/* Not sure if useful at all anymore */
 #define CONFIG_GRAD_COLOR   26
-#define CONFIG_NUMS	    27
+#define CONFIG_RESISTS      27
+#define CONFIG_SMOOTH       28
+#define CONFIG_SPLASH	    29
+#define CONFIG_APPLY_CONTAINER	30	/* Reapply container */
+#define CONFIG_NUMS	    31
 
 /* CONFIG_LIGHTING can have several possible values - set them accordingly */
 #define CFG_LT_TILE	    1
@@ -176,7 +189,7 @@ typedef struct Stat_struct {
     sint16 maxsp;	    /* Max spell points. */
     sint16 grace;	    /* Spell points.  Used to cast spells. */
     sint16 maxgrace;	    /* Max spell points. */
-    sint32 exp;		    /* Experience.  Killers gain 1/10. */
+    sint64 exp;		    /* Experience.  Killers gain 1/10. */
     sint16 food;	    /* How much food in stomach.  0 = starved. */
     sint16 dam;		    /* How much damage this object does when hitting */
     sint32 speed;	    /* Gets converted to a float for display*/
@@ -185,7 +198,7 @@ typedef struct Stat_struct {
     sint16 resists[30];	    /* Resistant values */
     uint32 resist_change:1; /* Resistant value has changed */
     sint16 skill_level[MAX_SKILL];  /* Level and experience totals for */
-    sint32 skill_exp[MAX_SKILL];    /* skills */
+    sint64 skill_exp[MAX_SKILL];    /* skills */
 } Stats;
 
 
@@ -230,6 +243,13 @@ typedef struct Player_Struct {
  */
 
 #define MAX_FACE_SETS   20
+
+/* Max size of image in each direction.  This
+ * is needed for the x11 client, which wants to
+ * initalize some data once.  Increasing this would
+ * likely only need a bigger footrpint
+ */
+#define MAX_IMAGE_SIZE 320
 typedef struct {
     uint8   setnum;
     char    *prefix;
@@ -266,8 +286,10 @@ extern Face_Information face_info;
 extern Client_Player cpl;		/* Player object. */
 extern char *skill_names[MAX_SKILL];
 
+#ifndef CPROTO
 /* We need to declare most of the structs before we can include this */
 #include <proto.h>
+#endif
 
 extern int errno;
 
@@ -285,6 +307,22 @@ extern int map1cmd,metaserver_on;
  */
 #define MAP_MAX_SIZE	31
 
+/* This is basically the smallest the map structure
+ * used for the client can be.  It needs to be bigger than
+ * the min map size above simply because we have to deal with
+ * off map big images, Also, we move the center point
+ * around within this map, so that if the player moves one space,
+ * we don't have to move around all the data.
+ */
+#define MIN_ALLOCATED_MAP_SIZE	MAP_MAX_SIZE * 2
+
+/* This is how many spaces an object might extend off the map.
+ * Eg, for bigimage stuff, the head of the image may be off the
+ * the map edge.  This is the most it may be off.  This is needed
+ * To cover case of need_recenter_map routines.
+ */
+#define MAX_MAP_OFFSET	6
+
 /* Fog of war stuff */
 #define FOG_MAP_SIZE 512   /* Default size of virtual map */
 
@@ -322,9 +360,11 @@ struct MapCellLayer {
 struct MapCell {
     struct MapCellLayer	heads[MAXLAYERS];
     struct MapCellLayer	tails[MAXLAYERS];
+    uint16 smooth[MAXLAYERS];
     uint8 darkness;
     uint8 need_update:1;
     uint8 have_darkness:1;
+    uint8 need_resmooth:1;  /*same has need update but for smoothing only*/
     uint8 cleared:1; /* If set, this is a fog cell. */
 };
 
