@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.7 2002/05/31 03:44:54 mwedel Exp $";
+ *   "$Id: commands.c,v 1.17 2003/10/10 06:18:52 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -64,10 +64,41 @@
  *   else as well.
  *   The unpacking routines basically perform the opposite operations.
  */
-
+int mapupdatesent=0;
 #include <client.h>
 #include <external.h>
 
+static void get_skill_info(char *data, int len)
+{
+    char *cp, *nl, *sn;
+    int val;
+
+    cp = data;
+    do {
+	nl = strchr(cp, '\n');
+	if (nl) {
+	    *nl=0;
+	    nl++;
+	}
+	sn = strchr(cp, ':');
+	if (!sn) {
+	    fprintf(stderr,"get_skill_info: corrupt line: %s\n", cp);
+	    return;
+	}
+	*sn=0;
+	sn++;
+	val = atoi(cp);
+	val -= CS_STAT_SKILLINFO;
+	if (val < 0 || val> CS_NUM_SKILLS) {
+	    fprintf(stderr,"get_skill_info: invalid skill number %d\n", val);
+	    return;
+	}
+	if (skill_names[val]) free(skill_names[val]);
+	skill_names[val] = strdup_local(sn);
+	cp = nl;
+    } while (cp < (data + len));
+}
+
 
 /* handles the response from a 'requestinfo' command.  This function doesn't
  * do much itself other than dispatch to other functions.
@@ -96,6 +127,9 @@ void ReplyInfoCmd(char *buf, int len)
     else if (!strcmp(buf,"image_sums")) {
 	get_image_sums(cp, len - i - 1);   /* located in common/image.c */
     }
+    else if (!strcmp(buf,"skill_info")) {
+	get_skill_info(cp, len - i - 1);   /* located in common/image.c */
+    }
 }
     
 
@@ -233,15 +267,62 @@ void SetupCmd(char *buf, int len)
 	     * are used in the future, we should probably fall back one at
 	     * a time or the like.
 	     */
+	} else if (!strcmp(cmd,"exp64")) {
+	    /* If this server does not support the new skill code,
+	     * fall back to the old which uses experience categories.
+	     * for that, initialize the skill names appropriately.
+	     * by doing so, the actual display code can be the same
+	     * for both old and new.  If the server does support
+	     * new exp system, send a request to get the mapping
+	     * information.
+	     */
+	    if (!strcmp(param,"FALSE")) {
+		int i;
+		for (i=0; i<MAX_SKILL; i++) {
+		    if (skill_names[i]) {
+			free(skill_names[i]);
+			skill_names[i] = NULL;
+		    }
+		}
+		skill_names[0] = strdup_local("agility");
+		skill_names[1] = strdup_local("personality");
+		skill_names[2] = strdup_local("mental");
+		skill_names[3] = strdup_local("physique");
+		skill_names[4] = strdup_local("magic");
+		skill_names[5] = strdup_local("wisdom");
+	    }
+	    else {
+		cs_print_string(csocket.fd,"requestinfo skill_info");
+	    }
 	}
-
+    else if (!strcmp(cmd,"extendedMapInfos")) {
+        if (!strcmp(param,"FALSE")) {
+            use_config[CONFIG_SMOOTH]=0;
+        }else{
+            /* Request all extended infos we want
+             * Should regroup everything for easyness
+             */
+            if (use_config[CONFIG_SMOOTH]){
+                cs_print_string(csocket.fd,"toggleextendedinfos smooth");
+            }    
+        }
+    }
 	else {
 		fprintf(stderr,"Got setup for a command we don't understand: %s %s\n",
 		    cmd, param);
 	}
     }
 }
-
+void ExtendedInfoSetCmd (char *data, int len){
+    (void)data;(void)len; /* __UNUSED__ */
+    /* Do nothing for now, perhaps later add some
+     * support to check what server knows.
+     */
+   /* commented, no waranty string data is null terminated
+   draw_info("ExtendedInfoSet returned from server: ",NDI_BLACK);
+   draw_info(data,NDI_BLACK);
+   */
+}
 
 /* Handles when the server says we can't be added.  In reality, we need to
  * close the connection and quit out, because the client is going to close
@@ -249,6 +330,7 @@ void SetupCmd(char *buf, int len)
  */
 void AddMeFail(char *data, int len)
 {
+    (void)data;(void)len; /* __UNUSED__ */
     LOG(0,"addme_failed received.\n");
     return;
 }
@@ -258,12 +340,14 @@ void AddMeFail(char *data, int len)
  */
 void AddMeSuccess(char *data, int len)
 {
+    (void)data;(void)len; /* __UNUSED__ */
     LOG(0,"addme_success received.\n");
     return;
 }
 
 void GoodbyeCmd(char *data, int len)
 {
+    (void)data;(void)len; /* __UNUSED__ */
     /* This could probably be greatly improved - I am not sure if anything
      * needs to be saved here, but certainly it should be possible to
      * reconnect to the server or a different server without having to
@@ -303,12 +387,64 @@ void AnimCmd(unsigned char *data, int len)
     LOG(0,"Received animation %d, %d faces\n", anum, animations[anum].num_animations);
 }
 
+Smooths smooths[MAXSMOOTH];
+int smoothused=0;
+void SmoothCmd(unsigned char *data, int len){
+    uint16 faceid;
+    uint16 smoothing;
+    static int dx[8]={0,1,1,1,0,-1,-1,-1};
+    static int dy[8]={-1,-1,0,1,1,1,0,-1};  
+    int i,j,x,y,layer;
+    int mx,my;
+
+    /* len is unused.
+     * We should check that we don't have an invalid short command.
+     * Hence, the compiler warning is valid.
+     */
+    display_map_startupdate();
+    if (smoothused>=MAXSMOOTH) /*no place to put*/
+        return;
+    faceid=GetShort_String(data);
+    smoothing=GetShort_String(data+2);
+    for (i=0;i<smoothused;i++){
+        if (smooths[i].smoothid==faceid)
+            break;
+    }
+    if (i==smoothused)
+        smoothused++;
+    smooths[i].smoothid=faceid;
+    smooths[i].received=1;
+    smooths[i].face=smoothing;    
+    /*update the map where needed*/
+    for( x= 0; x<use_config[CONFIG_MAPWIDTH]; x++) {
+    for( y= 0; y<use_config[CONFIG_MAPHEIGHT]; y++){
+    for (layer=0;layer<MAXLAYERS;layer++){
+        mx = x + pl_pos.x;
+	    my = y + pl_pos.y;
+        if ( (the_map.cells[mx][my].smooth[layer]==0) ||
+             (the_map.cells[mx][my].heads[layer].face!=smooths[i].smoothid))
+            continue; /*not concerned*/
+        for (j=0;j<8;j++){
+            mx=x + pl_pos.x+dx[j];
+            my=y + pl_pos.y+dy[j];
+            if ( (mx<0) || (my<0) || (the_map.x<=mx) || (the_map.y<=my))
+                continue;
+            if (!the_map.cells[mx][my].cleared)
+                the_map.cells[mx][my].need_resmooth=1;   
+        }
+    }
+    }
+    }
+    display_map_doneupdate(FALSE);
+    /*Should find some way to draw here, i suppose*/
+}
 
 void DrawInfoCmd(char *data, int len)
 {
     int color=atoi(data);
     char *buf;
 
+    (void)len; /* __UNUSED__ */
     buf = strchr(data, ' ');
     if (!buf) {
 	fprintf(stderr,"DrawInfoCmd - got no data\n");
@@ -324,8 +460,8 @@ void DrawInfoCmd(char *data, int len)
 
 void StatsCmd(unsigned char *data, int len)
 {
-  int i=0;
-  int c;
+    int i=0, c, redraw=0;
+    sint64 last_exp;
 
     while (i<len) {
 	c=data[i++];
@@ -333,31 +469,47 @@ void StatsCmd(unsigned char *data, int len)
 	    cpl.stats.resists[c-CS_STAT_RESIST_START]=GetShort_String(data+i);
 	    i+=2;
 	    cpl.stats.resist_change=1;
+	} else if (c >= CS_STAT_SKILLINFO && c < (CS_STAT_SKILLINFO+CS_NUM_SKILLS)) {
+	    /* We track to see if the exp has gone from 0 to some total value -
+	     * we do this because the draw logic currently only draws skills where
+	     * the player has exp.  We need to communicate to the draw function
+	     * that it should draw all the players skills.  Using redraw is
+	     * a little overkill, because a lot of the data may not be changing.
+	     * OTOH, such a transition should only happen rarely, not not be a very
+	     * big deal.
+	     */
+	    cpl.stats.skill_level[c - CS_STAT_SKILLINFO] = data[i++];
+	    last_exp = cpl.stats.skill_exp[c - CS_STAT_SKILLINFO];
+	    cpl.stats.skill_exp[c - CS_STAT_SKILLINFO] = GetInt64_String(data+i);
+	    if (last_exp == 0 && cpl.stats.skill_exp[c - CS_STAT_SKILLINFO])
+		redraw=1;
+	    i += 8;
 	} else {
 	    switch (c) {
 		case CS_STAT_HP:	cpl.stats.hp=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_MAXHP:cpl.stats.maxhp=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_MAXHP:	cpl.stats.maxhp=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_SP:	cpl.stats.sp=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_MAXSP:cpl.stats.maxsp=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_GRACE:cpl.stats.grace=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_MAXGRACE:cpl.stats.maxgrace=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_MAXSP:	cpl.stats.maxsp=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_GRACE:	cpl.stats.grace=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_MAXGRACE:	cpl.stats.maxgrace=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_STR:	cpl.stats.Str=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_INT:	cpl.stats.Int=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_POW:	cpl.stats.Pow=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_WIS:	cpl.stats.Wis=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_DEX:	cpl.stats.Dex=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_CON:	cpl.stats.Con=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_CHA:  cpl.stats.Cha=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_EXP:  cpl.stats.exp=GetInt_String(data+i); i+=4; break;
-		case CS_STAT_LEVEL:cpl.stats.level=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_WC:   cpl.stats.wc=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_AC:   cpl.stats.ac=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_DAM:  cpl.stats.dam=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_ARMOUR:cpl.stats.resists[0]=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_SPEED: cpl.stats.speed=GetInt_String(data+i); i+=4; break;
-		case CS_STAT_FOOD:  cpl.stats.food=GetShort_String(data+i); i+=2; break;
-		case CS_STAT_WEAP_SP:cpl.stats.weapon_sp=GetInt_String(data+i); i+=4; break;
-		case CS_STAT_FLAGS:cpl.stats.flags=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_CHA:	cpl.stats.Cha=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_EXP:	cpl.stats.exp=GetInt_String(data+i); i+=4; break;
+		case CS_STAT_EXP64:	cpl.stats.exp=GetInt64_String(data+i); i+=8; break;
+		case CS_STAT_LEVEL:	cpl.stats.level=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_WC:	cpl.stats.wc=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_AC:	cpl.stats.ac=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_DAM:	cpl.stats.dam=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_ARMOUR:    cpl.stats.resists[0]=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_SPEED:	cpl.stats.speed=GetInt_String(data+i); i+=4; break;
+		case CS_STAT_FOOD:	cpl.stats.food=GetShort_String(data+i); i+=2; break;
+		case CS_STAT_WEAP_SP:	cpl.stats.weapon_sp=GetInt_String(data+i); i+=4; break;
+		case CS_STAT_FLAGS:	cpl.stats.flags=GetShort_String(data+i); i+=2; break;
 		case CS_STAT_WEIGHT_LIM:set_weight_limit(GetInt_String(data+i)); i+=4; break;
 
 		/* Skill experience handling */
@@ -404,11 +556,12 @@ void StatsCmd(unsigned char *data, int len)
 	    }
 	}
     }
+
     if (i>len) {
 	fprintf(stderr,"got stats overflow, processed %d bytes out of %d\n",
 		i, len);
     }
-    draw_stats(0);
+    draw_stats(redraw);
     draw_message_window(0);
 }
 
@@ -417,6 +570,7 @@ void handle_query (char *data, int len)
     char *buf,*cp;
     uint8 flags = atoi(data);
 
+    (void)len; /* __UNUSED__ */
     /* The actual text is optional */
     buf = strchr(data,' ');
     if (buf) buf++;
@@ -474,6 +628,7 @@ void PlayerCmd(unsigned char *data, int len)
     char name[MAX_BUF];
     int tag, weight, face,i=0,nlen;
 
+    reset_player_data();
     tag = GetInt_String(data);
     i+=4;
     weight = GetInt_String(data+i);
@@ -502,11 +657,6 @@ void item_actions (item *op)
 	close_container (op);
 	cpl.container=NULL;
     }
-/*
-    if (op->env == cpl.below) {
-	check_auto_pickup (op);
-    }
-*/
 }
 
 
@@ -665,8 +815,10 @@ void DeleteItem(unsigned char *data, int len)
 void DeleteInventory(unsigned char *data, int len)
 {
     int tag;
+    (void)len; /* __UNUSED__ */
 
     tag=atoi((const char*)data);
+
     if (tag<0) {
 	fprintf(stderr,"DeleteInventory: Invalid tag: %d\n", tag);
 	return;
@@ -737,6 +889,23 @@ static void expand_face(int hx, int hy, int dx, int dy, int oldface, int newface
 		return;
 	    }
 
+	    /* the layer we were supposed to go on is filled by some other object.
+	     * In this case, we want to see what object is more to the lower right - that
+	     * should go on the higher layer so that overlaps look proper.
+	     * Not positive if the rationale here is correct, but what I have is the 
+	     * y axis is most important.  If y is equal, then further to the right
+	     * is more important
+	     */
+	    if (l > layer) {
+		if ((the_map.cells[zx][zy].tails[layer].size_y > y) ||
+		    (the_map.cells[zx][zy].tails[layer].size_y == y && (the_map.cells[zx][zy].tails[layer].size_x > x))) {
+			the_map.cells[zx][zy].tails[l].size_x = the_map.cells[zx][zy].tails[layer].size_x;
+			the_map.cells[zx][zy].tails[l].size_y = the_map.cells[zx][zy].tails[layer].size_y;
+			the_map.cells[zx][zy].tails[l].face = the_map.cells[zx][zy].tails[layer].face;
+			l = layer;
+		    }
+	    }
+
 	    the_map.cells[zx][zy].tails[l].face = newface;
 	    /* only set the size_ values if there is an actual image */
 	    if (newface) {
@@ -844,6 +1013,9 @@ void display_map_clearcell(int x,int y)
  */
 static void set_map_face(int x, int y, int layer, int face)
 {
+    static int dx[8]={0,1,1,1,0,-1,-1,-1};
+    static int dy[8]={-1,-1,0,1,1,1,0,-1};
+    int rx,ry,i;
     x+= pl_pos.x;
     y+= pl_pos.y;
 
@@ -866,6 +1038,17 @@ static void set_map_face(int x, int y, int layer, int face)
 		    &the_map.cells[x][y].heads[layer].size_x, 
 		    &the_map.cells[x][y].heads[layer].size_y);
 
+    /*If this square has a smoothlevel, need to resmooth adjacent squares too*/
+    if (use_config[CONFIG_SMOOTH]){
+        if (the_map.cells[x][y].smooth[layer]>0)
+            for (i=0;i<8;i++){
+                rx=x+dx[i];
+                ry=y+dy[i];
+                if ( (rx<0) || (ry<0) || (the_map.x<=rx) || (the_map.y<=ry))
+                    continue;
+                the_map.cells[rx][ry].need_resmooth=1;  
+            }
+    }
     the_map.cells[x][y].need_update = 1;
     the_map.cells[x][y].have_darkness = 1;
 }
@@ -873,10 +1056,9 @@ static void set_map_face(int x, int y, int layer, int face)
 
 void NewmapCmd(unsigned char *data, int len)
 {
+    (void)data;(void)len; /* __UNUSED__ */
     display_map_newmap();
 }
-
-
 /* This is the common processing block for the map1 and
  * map1a protocol commands.  The map1a mieks minor extensions
  * and are easy to deal with inline (in fact, this code
@@ -892,7 +1074,8 @@ static void map1_common(unsigned char *data, int len, int rev)
     int mask, x, y, pos=0,face, layer;
 
     map1cmd=1;
-    display_map_startupdate();
+    if (!mapupdatesent) /* see MapExtendedCmd */
+        display_map_startupdate();
 /*    fprintf(stderr,"map1 bytes %d\n", len);*/
 
     while (pos <len) {
@@ -928,6 +1111,10 @@ static void map1_common(unsigned char *data, int len, int rev)
 	for (layer=NUM_LAYERS; layer>=0; layer--) {
 	    if (mask & (1 << layer)) {
 		face = GetShort_String(data+pos); pos +=2;
+        if ( (face<0) || (face>10000)){
+            printf ("BUG: face %d for %dx%d at layer %d\n",face,x,y,NUM_LAYERS - layer);
+            printf ("     mask was %2X\n",mask);
+        }
 		set_map_face(x, y, NUM_LAYERS - layer, face);
 	    }
 	}
@@ -949,6 +1136,7 @@ static void map1_common(unsigned char *data, int len, int rev)
 		    0, the_map.cells[x][y].heads[layer].face, layer);
 	}
     }
+    mapupdatesent=0;
     display_map_doneupdate(FALSE);
 
 }
@@ -973,6 +1161,7 @@ void map_scrollCmd(char *data, int len)
     int dx,dy;
     char *buf;
 
+    (void)len; /* __UNUSED__ */
     dx = atoi(data);
     buf = strchr(data,' ');
     if (!buf) {
@@ -984,6 +1173,93 @@ void map_scrollCmd(char *data, int len)
     display_mapscroll(dx,dy);
 }
 
+/* Extract smoothing infos from an extendedmapinfo packet part
+ * data is located at the beginning of the smooth datas
+ */
+int ExtSmooth(unsigned char* data,int len,int x,int y,int layer){
+    static int dx[8]={0,1,1,1,0,-1,-1,-1};
+    static int dy[8]={-1,-1,0,1,1,1,0,-1};
+    int i,rx,ry;
+    int newsm;
+    if (len<1)
+        return 0;
+    x+= pl_pos.x;
+	y+= pl_pos.y;
+    newsm=GetChar_String(data);
+    if (the_map.cells[x][y].smooth[layer]!=newsm)
+    for (i=0;i<8;i++){
+        rx=x+dx[i];
+        ry=y+dy[i];
+        if ( (rx<0) || (ry<0) || (the_map.x<=rx) || (the_map.y<=ry))
+            continue;
+        the_map.cells[x][y].need_resmooth=1;            
+    }
+    the_map.cells[x][y].smooth[layer]=newsm;
+    return 1;/*Cause smooth infos only use 1 byte*/
+}
+/* Handle MapExtended command
+ * Warning! if you add commands to extended, take
+ * care that the 'layer' argument of main loop is
+ * the opposite of the layer of the map.
+ * so if you reference a layer, use NUM_LAYERS-layer
+ */
+void MapExtendedCmd(unsigned char *data, int len){
+    int mask, x, y, pos=0, layer;
+    int noredraw=0;
+    int hassmooth=0;
+    int entrysize;
+    int startpackentry;
+    map1cmd=1;
+    if (!mapupdatesent)
+        display_map_startupdate();
+    mapupdatesent=1;
+	mask = GetChar_String(data+pos); pos+=1;
+    if (mask&EMI_NOREDRAW)
+        noredraw=1;
+    if (mask&EMI_SMOOTH){
+        hassmooth=1;
+    }
+    while (mask&EMI_HASMOREBITS){
+        /*There may be bits we ignore about*/
+        mask = GetChar_String(data+pos); 
+        pos+=1;
+    }
+    entrysize=GetChar_String(data+pos); 
+    pos=pos+1;
+    
+    
+    while (pos+entrysize+2 <=len) {
+        mask = GetShort_String(data+pos); pos+=2;
+        x = (mask >>10) & 0x3f;
+        y = (mask >>4) & 0x3f;       
+        for (layer=NUM_LAYERS; layer>=0; layer--) {
+            if (mask & (1 << layer)) {
+                /*handle an entry*/
+                if (pos+entrysize>len)/*erroneous packet*/
+                    break;
+                startpackentry=pos;
+                /* If you had extended infos to the server, this
+                 * is where, in the client, you may add your code
+                 */
+                if (hassmooth)
+                    pos=pos+ExtSmooth(data+pos,len-pos,x,y,NUM_LAYERS - layer);
+                /* continue with other if you add new extended 
+                 * infos to server
+                 */
+                 
+                /* Now point to the next data */
+                pos=startpackentry+entrysize; 
+                
+            }
+        }
+	}
+    if (!noredraw){        
+        display_map_doneupdate(FALSE);
+        mapupdatesent=0;
+    }
+}
+
+
 void MagicMapCmd(unsigned char *data, int len)
 {
     unsigned char *cp;
