@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.5 2002/08/14 04:37:56 mwedel Exp $";
+ *   "$Id: item.c,v 1.6 2003/10/10 06:18:52 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -416,6 +416,16 @@ item *create_new_item (item *env, sint32 tag)
     return op;
 }
 
+int num_free_items()
+{
+    item *tmp;
+    int count=0;
+
+    for (tmp=free_items; tmp; tmp=tmp->next)
+	count++;
+    return count;
+}
+
 /*
  *  Hardcoded now, server could send these at initiation phase.
  */
