@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newsocket_c =
- *   "$Id: newsocket.c,v 1.3 2002/01/15 07:32:59 mwedel Exp $";
+ *   "$Id: newsocket.c,v 1.6 2003/06/19 12:23:39 tchize Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -132,6 +132,10 @@ int SockList_Send(SockList *sl, int fd)
 }
 
 
+char GetChar_String(unsigned char *data)
+{
+    return (data[0]);
+}
 /* Basically does the reverse of SockList_AddInt, but on
  * strings instead.  Same for the GetShort, but for 16 bits.
  */
@@ -140,6 +144,14 @@ int GetInt_String(unsigned char *data)
     return ((data[0]<<24) + (data[1]<<16) + (data[2]<<8) + data[3]);
 }
 
+/* 64 bit version of the above */
+sint64 GetInt64_String(unsigned char *data)
+{
+    return (((uint64)data[0]<<56) + ((uint64)data[1]<<48) + 
+	    ((uint64)data[2]<<40) + ((uint64)data[3]<<32) +
+	    (data[4]<<24) + (data[5]<<16) + (data[6]<<8) + data[7]);
+}
+
 short GetShort_String(unsigned char *data) {
     return ((data[0]<<8)+data[1]);
 }
@@ -167,7 +179,8 @@ int SockList_ReadPacket(int fd, SockList *sl, int len)
 	     */
 	    if (errno!=EAGAIN && errno!=EWOULDBLOCK) {
 		perror("ReadPacket got an error.");
-		LOG(llevDebug,"ReadPacket got error %d, returning 0",errno);
+		LOG(llevDebug,"ReadPacket got error %d, returning -1",errno);
+		return -1;
 	    }
 	    return 0;	/*Error */
 	}
