@@ -5,10 +5,12 @@ extern void negotiate_connection(int sound);
 /* commands.c */
 extern void ReplyInfoCmd(char *buf, int len);
 extern void SetupCmd(char *buf, int len);
+extern void ExtendedInfoSetCmd(char *data, int len);
 extern void AddMeFail(char *data, int len);
 extern void AddMeSuccess(char *data, int len);
 extern void GoodbyeCmd(char *data, int len);
 extern void AnimCmd(unsigned char *data, int len);
+extern void SmoothCmd(unsigned char *data, int len);
 extern void DrawInfoCmd(char *data, int len);
 extern void StatsCmd(unsigned char *data, int len);
 extern void handle_query(char *data, int len);
@@ -26,9 +28,13 @@ extern void NewmapCmd(unsigned char *data, int len);
 extern void Map1Cmd(unsigned char *data, int len);
 extern void Map1aCmd(unsigned char *data, int len);
 extern void map_scrollCmd(char *data, int len);
+extern int ExtSmooth(unsigned char *data, int len, int x, int y, int layer);
+extern void MapExtendedCmd(unsigned char *data, int len);
 extern void MagicMapCmd(unsigned char *data, int len);
 /* image.c */
 extern void init_common_cache_data(void);
+extern void requestsmooth(int pnum);
+extern int getsmooth(int face);
 extern void requestface(int pnum, char *facename);
 extern void finish_face_cmd(int pnum, uint32 checksum, int has_sum, char *face, int faceset);
 extern void reset_image_cache_data(void);
@@ -46,6 +52,7 @@ extern void SendVersion(ClientSocket csock);
 extern void SendAddMe(ClientSocket csock);
 extern void SendSetFaceMode(ClientSocket csock, int mode);
 extern void init_client_vars(void);
+extern void reset_player_data(void);
 extern void reset_client_vars(void);
 /* item.c */
 extern void init_item_types(void);
@@ -57,6 +64,7 @@ extern item *locate_item(sint32 tag);
 extern void remove_item(item *op);
 extern void remove_item_inventory(item *op);
 extern item *create_new_item(item *env, sint32 tag);
+extern int num_free_items(void);
 extern void set_item_values(item *op, char *name, sint32 weight, uint16 face, uint16 flags, uint16 anim, uint16 animspeed, sint32 nrof, uint16 type);
 extern void toggle_locked(item *op);
 extern void send_mark_obj(item *op);
@@ -81,7 +89,9 @@ extern void SockList_AddShort(SockList *sl, uint16 data);
 extern void SockList_AddInt(SockList *sl, uint32 data);
 extern void SockList_AddString(SockList *sl, const char *str);
 extern int SockList_Send(SockList *sl, int fd);
+extern char GetChar_String(unsigned char *data);
 extern int GetInt_String(unsigned char *data);
+extern sint64 GetInt64_String(unsigned char *data);
 extern short GetShort_String(unsigned char *data);
 extern int SockList_ReadPacket(int fd, SockList *sl, int len);
 extern int cs_print_string(int fd, char *str, ...);
@@ -101,6 +111,6 @@ extern void stop_run(void);
 extern void run_dir(int dir);
 extern int send_command(const char *command, int repeat, int must_send);
 extern void CompleteCmd(unsigned char *data, int len);
-extern void show_help(void);
+extern void command_take(const char *command, char *cpnext);
 extern void extended_command(const char *ocommand);
 extern char *complete_command(char *command);
