@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_gx11_h =
- *   "$Id: gx11.h,v 1.6 2002/05/22 04:58:26 mwedel Exp $";
+ *   "$Id: gx11.h,v 1.8 2003/09/13 05:25:07 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -55,6 +55,7 @@ typedef struct {
     uint16	icon_width, icon_height;
     void	*map_mask, *map_image;
     uint16	map_width, map_height;
+    void	*fog_image;
 } PixmapInfo;
 
 extern PixmapInfo *pixmaps[MAXPIXMAPNUM];
@@ -135,5 +136,9 @@ typedef struct {
 
 extern itemlist inv_list, look_list;
 
+/* This was just a test I put in to try different redraw methods */
+#define ALTERNATE_MAP_REDRAW	0
+
 
 #endif
+
