@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.10 2002/07/03 21:10:54 mwedel Exp $";
+ *   "$Id: map.c,v 1.16 2003/08/08 17:06:40 tchize Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -355,9 +355,16 @@ void display_mapscroll(int dx,int dy)
 		if( x < 0 || y < 0 || x >= the_map.x ||	y >= the_map.y)
 		    continue;
 
-		    
+
 		the_map.cells[x][y].need_update= 1;
-		the_map.cells[x][y].cleared= 1;
+        /*After long long long investigations on why the multipart objects did
+          disappear when entering map view from right or bottom, scrolling
+          through whole server code, i concluded the following line should be
+          commented. If a multipart object was on a square outside of map,
+          containing the tail, tail may be cleared by following process and
+          so we end up with things like tower disappearance.
+                    tchize@myrealbox.com*/
+		/*the_map.cells[x][y].cleared= 1;*/
 	    }
 	} /* for y */
     } /* for x */
@@ -365,7 +372,7 @@ void display_mapscroll(int dx,int dy)
 #ifdef HAVE_SDL
     if (use_config[CONFIG_SDL])
 	sdl_mapscroll(dx,dy);
-    else 
+    else
 #endif
 	map_did_scroll = 1;
 /*    fprintf(stderr,"scroll command: %d %d\n", dx, dy);*/
@@ -396,23 +403,130 @@ void reset_map_data()
     }
 }
 
-/* This draws the map using GdkPixmaps/GdkBitmaps.  It is based off of sdl_gen_map.
- * We draw on a GdkPixmap drawable (mapwindow), which we then copy to the 
- * visible area on the screen.  Doing this reduces flickering that otherwise happen
- * if we draw directly to the window.
- * The performance of this function seems to be very good - presumably because
- * most of this data is stored in the x-server in a format native to it.  Also,
- * per pixel manipulations are not done - either the entire pixel is copied,
- * or it is not, no blending is done.  This means that true alpha blending
- * is not supported, but nothing uses that now anyways.
+         
+/* Draw anything in adjacent squares that could smooth on given square
+ * mx,my square to smooth on. you should not call this function to
+ * smooth on a 'completly black' square.
+ * layer layer to examine (we smooth only one layer at a time)
+ * picx,picy place on the mapwindow to draw
  */
-
-
+void drawsmooth (int mx,int my,int layer,int picx,int picy){
+    static int dx[8]={0,1,1,1,0,-1,-1,-1};
+    static int dy[8]={-1,-1,0,1,1,1,0,-1};
+    static int bweights[8]={2,0,4,0,8,0,1,0};
+    static int cweights[8]={0,2,0,4,0,8,0,1};
+    static int bc_exclude[8]={
+                 1+2,/*north exclude northwest (bit0) and northeast(bit1)*/
+                 0,
+                 2+4,/*east exclude northeast and southeast*/
+                 0,
+                 4+8,/*and so on*/
+                 0,
+                 8+1,
+                 0
+                };
+    int partdone[8]={0,0,0,0,0,0,0,0};
+    int slevels[8];
+    int sfaces[8];
+    int i,lowest,weight,weightC;
+    int emx,emy;
+    int smoothface;
+    int dosmooth=0;
+    if (the_map.cells[mx][my].heads[0].face==0)
+        return;
+    for (i=0;i<8;i++){
+        emx=mx+dx[i];
+        emy=my+dy[i];
+        if ( (emx<0) || (emy<0) || (the_map.x<=emx) || (the_map.y<=emy)){
+            slevels[i]=0;
+            sfaces[i]=0; /*black picture*/
+        }
+        if (the_map.cells[emx][emy].smooth[layer]<=the_map.cells[mx][my].smooth[layer]){            
+            slevels[i]=0;
+            sfaces[i]=0; /*black picture*/
+        }else{      
+            slevels[i]=the_map.cells[emx][emy].smooth[layer];
+            sfaces[i]=getsmooth(the_map.cells[emx][emy].heads[layer].face);
+            dosmooth=1;
+        }                    
+    }
+    /* ok, now we have a list of smoothlevel higher than current square.
+     * there are at most 8 different levels. so... let's check 8 times
+     * for the lowest one (we draw from botto to top!).
+     */
+    lowest=-1;
+    while (1){
+        lowest = -1;
+        for (i=0;i<8;i++){
+            if ( (slevels[i]>0) && (!partdone[i]) &&
+                ((lowest<0) || (slevels[i]<slevels[lowest]))
+               )
+                    lowest=i;    
+        }
+        if (lowest<0)
+            break;   /*no more smooth to do on this square*/
+        /*printf ("hey, must smooth something...%d\n",sfaces[lowest]);*/
+        /*here we know 'what' to smooth*/
+        /* we need to calculate the weight
+         * for border and weight for corners.
+         * then we 'markdone' 
+         * the corresponding squares
+         */
+        /*first, the border, which may exclude some corners*/
+        weight=0;
+        weightC=15; /*works in backward. remove where there is nothing*/
+        /*for (i=0;i<8;i++)
+            cornermask[i]=1;*/
+        for (i=0;i<8;i++){ /*check all nearby squares*/
+            if ( (slevels[i]==slevels[lowest]) &&
+                 (sfaces[i]==sfaces[lowest])){
+                partdone[i]=1;
+                weight=weight+bweights[i];
+                weightC&=~bc_exclude[i];
+            }else{
+                /*must rmove the weight of a corner if not in smoothing*/
+                weightC&=~cweights[i];
+            }
+            
+        }
+        /*We can't do this before since we need the partdone to be adjusted*/
+        if (sfaces[lowest]<=0)
+            continue;  /*Can't smooth black*/
+        smoothface=sfaces[lowest];
+        if (smoothface<=0){
+            continue;  /*picture for smoothing not yet available*/
+        }
+        /* now, it's quite easy. We must draw using a 32x32 part of
+         * the picture smoothface. 
+         * This part is located using the 2 weights calculated: 
+         * (32*weight,0) and (32*weightC,32)
+         */
+        if ( (!pixmaps[smoothface]->map_image) ||
+             (pixmaps[smoothface] == pixmaps[0]))
+            continue;   /*don't have the picture associated*/
+        if (weight>0){
+        gdk_gc_set_clip_mask (mapgc, pixmaps[smoothface]->map_mask);
+         
+        gdk_gc_set_clip_origin(mapgc, picx-map_image_size*weight,picy);
+        gdk_draw_pixmap(mapwindow, mapgc,pixmaps[smoothface]->map_image,
+	        map_image_size*weight, 0, picx, picy,
+            map_image_size, map_image_size);
+        }
+        if (weightC>0){
+        gdk_gc_set_clip_mask (mapgc, pixmaps[smoothface]->map_mask);
+        gdk_gc_set_clip_origin(mapgc, picx-map_image_size*weightC,picy-map_image_size);
+        gdk_draw_pixmap(mapwindow, mapgc,pixmaps[smoothface]->map_image,
+		  map_image_size*weightC, map_image_size, picx, picy,
+          map_image_size, map_image_size);          
+        }
+		
+            
+    }/*while there's some smooth to do*/
+}
 void gtk_draw_map(int redraw) {
     int mx,my, layer,x,y, src_x, src_y;
     struct timeval tv1, tv2,tv3;
     long elapsed1, elapsed2;
-
     if (time_map_redraw)
 	gettimeofday(&tv1, NULL);
 
@@ -425,8 +539,8 @@ void gtk_draw_map(int redraw) {
 	    my = y + pl_pos.y;
 
 	    /* Don't need to touch this space */
-	    if (!redraw && !the_map.cells[mx][my].need_update && !map_did_scroll) continue;
-
+	    if (!redraw && !the_map.cells[mx][my].need_update && !map_did_scroll&& !the_map.cells[mx][my].need_resmooth)
+            continue;
 	    /* First, we need to black out this space. */
 	    gdk_draw_rectangle(mapwindow, drawingarea->style->black_gc, TRUE, x * map_image_size, y * map_image_size, map_image_size, map_image_size);
 	    /* now draw the different layers.  Only draw if using fog of war or the
@@ -460,13 +574,19 @@ void gtk_draw_map(int redraw) {
 		     * so we don't need to adjust the origin as much.
 		     */
 		    if (the_map.cells[mx][my].heads[layer].face && 
-			pixmaps[the_map.cells[mx][my].heads[layer].face]->map_image) {
+			pixmaps[the_map.cells[mx][my].heads[layer].face]->map_image){
 
 			/* add one to the size values to take into account the actual width of the space */
 			src_x = pixmaps[the_map.cells[mx][my].heads[layer].face]->map_width - map_image_size;
 			src_y = pixmaps[the_map.cells[mx][my].heads[layer].face]->map_height - map_image_size;
 
-			gdk_gc_set_clip_mask (mapgc, pixmaps[the_map.cells[mx][my].heads[layer].face]->map_mask);
+            
+            gdk_gc_set_clip_mask (mapgc, pixmaps[the_map.cells[mx][my].heads[layer].face]->map_mask);
+			/*GdkBitmap *newmask=NULL;
+            newmask=createpartialmask(pixmaps[the_map.cells[mx][my].heads[layer].face]->map_mask,
+                map_image_size/4,map_image_size/4,map_image_size/2,map_image_size/2,
+                map_image_size,map_image_size);
+            gdk_gc_set_clip_mask (mapgc, newmask);*/
 			gdk_gc_set_clip_origin(mapgc, 
 					       (x + 1 - the_map.cells[mx][my].heads[layer].size_x) * map_image_size,
 					       (y + 1 - the_map.cells[mx][my].heads[layer].size_y) * map_image_size);
@@ -475,9 +595,24 @@ void gtk_draw_map(int redraw) {
 				    pixmaps[the_map.cells[mx][my].heads[layer].face]->map_image,
 				    src_x, src_y, x * map_image_size, y * map_image_size,
 					map_image_size, map_image_size);
+            /*We draw the pic. Now, smooth adjacent squares on it*/
+            if ( use_config[CONFIG_SMOOTH])
+                drawsmooth (mx,my,layer,x * map_image_size,y * map_image_size);
+            }
+            /*Sometimes, it may happens we need to draw the smooth while there
+              is nothing to draw at that layer (but there was something at lower
+              layers). This is handled here. The else part is to take into account
+              cases where the smooth as already been handled 2 code lines before*/
+            else if ( use_config[CONFIG_SMOOTH] && 
+                 the_map.cells[mx][my].need_resmooth )
+                drawsmooth (mx,my,layer,x * map_image_size,y * map_image_size);
+            
+            /*if (newmask)
+                gdk_pixmap_unref(newmask);*/
 
-		    }
+		    
 		} /* else for processing the layers */
+        the_map.cells[mx][my].need_resmooth=0;
 
 	    /* Do final logic for this map space */
 	    the_map.cells[mx][my].need_update=0;
@@ -512,10 +647,10 @@ void gtk_draw_map(int redraw) {
 				x * map_image_size, y*map_image_size, map_image_size, map_image_size);
 	    }
 	    /* Don't redraw this space if we're going to redraw the entire map below */
-	    if (!map_did_scroll) 
+	    /*if (!map_did_scroll) 
 		gdk_draw_pixmap(drawingarea->window, drawingarea->style->black_gc, mapwindow, 
 			    x * map_image_size, y*map_image_size, x * map_image_size, y * map_image_size,
-			    map_image_size, map_image_size);
+			    map_image_size, map_image_size);*/
 
 	} /* For y spaces */
     } /* for x spaces */
@@ -531,7 +666,8 @@ void gtk_draw_map(int redraw) {
 		    0, 0, 0, 0, use_config[CONFIG_MAPWIDTH] * map_image_size, use_config[CONFIG_MAPHEIGHT] * map_image_size);
 	map_did_scroll = 0;
     }
-
+    else gdk_draw_pixmap(drawingarea->window, drawingarea->style->black_gc, mapwindow,
+		    0, 0, 0, 0, use_config[CONFIG_MAPWIDTH] * map_image_size, use_config[CONFIG_MAPHEIGHT] * map_image_size);
 
     if (time_map_redraw) {
 	gettimeofday(&tv3, NULL);
