@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_png_c =
- *   "$Id: png.c,v 1.4 2002/01/15 07:33:00 mwedel Exp $";
+ *   "$Id: png.c,v 1.6 2003/06/19 12:23:40 tchize Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -354,11 +354,16 @@ uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
 		a = xrow[3+source_column*BPP];
 	}
 
-	if (!needcol) {
-		nrows[destrow][dest_column * BPP] = r;
-		nrows[destrow][1+dest_column * BPP] = g;
-		nrows[destrow][2+dest_column * BPP] = b;
-		nrows[destrow][3+dest_column * BPP] = a;
+	/* Not positve, but without the bound checking for dest_column,
+	 * we were overrunning the buffer.  My guess is this only really
+	 * showed up if when the images are being scaled - there is probably
+	 * something like half a pixel left over.
+	 */
+	if (!needcol && (dest_column < new_width)) {
+	    nrows[destrow][dest_column * BPP] = r;
+	    nrows[destrow][1+dest_column * BPP] = g;
+	    nrows[destrow][2+dest_column * BPP] = b;
+	    nrows[destrow][3+dest_column * BPP] = a;
 	}
 	destrow++;
     }
@@ -390,8 +395,7 @@ int rgba_to_gdkpixmap(GdkWindow *window, uint8 *data,int width, int height,
 
     *mask=gdk_pixmap_new(window, width, height,1);
     gc_alpha=gdk_gc_new(*mask);
-    gdk_gc_set_function(gc_alpha, GDK_COPY);
-
+    
     scolor.pixel=1;
     gdk_gc_set_foreground(gc_alpha, &scolor);
     gdk_draw_rectangle(*mask, gc_alpha, 1, 0, 0, width, height);
