@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_sdl_c =
- *   "$Id: sdl.c,v 1.7 2002/07/03 21:10:54 mwedel Exp $";
+ *   "$Id: sdl.c,v 1.9 2003/09/13 05:25:07 mwedel Exp $";
  */
 
 /*
@@ -556,6 +556,13 @@ void sdl_gen_map(int redraw) {
     if (time_map_redraw)
 	gettimeofday(&tv1, NULL);
 
+#if !ALTERNATE_MAP_REDRAW
+    if (map_did_scroll) {
+	redraw=1;
+	map_did_scroll=0;
+    }
+#endif
+
     for( x= 0; x<use_config[CONFIG_MAPWIDTH]; x++) {
 	for(y = 0; y<use_config[CONFIG_MAPHEIGHT]; y++) {
 	    /* mx,my represent the spaces on the 'virtual' map (ie, the_map structure).
@@ -591,9 +598,15 @@ void sdl_gen_map(int redraw) {
 			src.h = map_image_size;
 			dst.x = x * map_image_size;
 			dst.y = y * map_image_size;
-			if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].tails[layer].face]->map_image, 
-			    &src, mapsurface, &dst))
-				do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			if (the_map.cells[mx][my].cleared) {
+			    if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].tails[layer].face]->fog_image, 
+				&src, mapsurface, &dst))
+				    do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			} else {
+			    if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].tails[layer].face]->map_image, 
+				&src, mapsurface, &dst))
+				    do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			}
 		    }
 		    /* Draw the head now - logic is pretty much exactly the same
 		     * as that for the tail, except we know that we this is at the lower right,
@@ -608,15 +621,22 @@ void sdl_gen_map(int redraw) {
 			src.h = map_image_size;
 			dst.x = x * map_image_size;
 			dst.y = y * map_image_size;
-			if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].heads[layer].face]->map_image, 
-			    &src, mapsurface, &dst))
-				do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			if (the_map.cells[mx][my].cleared) {
+			    if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].heads[layer].face]->fog_image, 
+				&src, mapsurface, &dst))
+				    do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			} else {
+			    if (SDL_BlitSurface(pixmaps[the_map.cells[mx][my].heads[layer].face]->map_image, 
+				&src, mapsurface, &dst))
+				    do_SDL_error( "BlitSurface", __FILE__, __LINE__);
+			}
 		    }
 		} /* else for processing the layers */
 
 	    /* Do final logic for this map space */
 	    the_map.cells[mx][my].need_update=0;
 
+#if 0
 	    if (the_map.cells[mx][my].cleared == 1 && use_config[CONFIG_FOGWAR]) {
 		/* If this is a fogcell, copy over the fogmap */
 		dst.x = x * map_image_size;
@@ -626,7 +646,9 @@ void sdl_gen_map(int redraw) {
 	    /* Only worry about lighting if it is not a fog cell.  If it is
 	     * a fog cell, lighting information is probably bogus.
 	     */
-	    else if (use_config[CONFIG_LIGHTING] == CFG_LT_TILE) {
+	    else 
+#endif
+	    if (use_config[CONFIG_LIGHTING] == CFG_LT_TILE) {
 		dst.x = x * map_image_size;
 		dst.y = y * map_image_size;
 		dst.w = map_image_size;
@@ -671,7 +693,6 @@ void sdl_mapscroll(int dx, int dy)
      * mapsurface->pitch is the length of a scanline in bytes 
      * including alignment padding
      */
-
     SDL_LockSurface( mapsurface);
     if( dy < 0) {
 	int offset= mapsurface->pitch * (-dy*map_image_size);
