@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_png_c =
- *   "$Id: png.c,v 1.8 2002/07/01 23:35:30 mwedel Exp $";
+ *   "$Id: png.c,v 1.10 2003/10/06 07:04:01 mwedel Exp $";
  */
 /*
     Crossfire client, a client program for the crossfire program.
@@ -225,8 +225,8 @@ uint8 *png_to_data(unsigned char *data, int len, int *width, int *height)
 
 uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
 {
-    static    int xrow[BPP * MAX_IMAGE_WIDTH], yrow[BPP*MAX_IMAGE_HEIGHT];
-    static  uint8  *nrows[MAX_IMAGE_HEIGHT];
+    static int xrow[BPP * MAX_IMAGE_WIDTH], yrow[BPP*MAX_IMAGE_HEIGHT];
+    static uint8 *nrows[MAX_IMAGE_HEIGHT];
 
     /* Figure out new height/width */
     int new_width = *width  * scale / RATIO, new_height = *height * scale / RATIO;
@@ -362,11 +362,16 @@ uint8 *rescale_rgba_data(uint8 *data, int *width, int *height, int scale)
 		a = xrow[3+source_column*BPP];
 	}
 
-	if (!needcol) {
-		nrows[destrow][dest_column * BPP] = r;
-		nrows[destrow][1+dest_column * BPP] = g;
-		nrows[destrow][2+dest_column * BPP] = b;
-		nrows[destrow][3+dest_column * BPP] = a;
+	/* Not positve, but without the bound checking for dest_column,
+	 * we were overrunning the buffer.  My guess is this only really
+	 * showed up if when the images are being scaled - there is probably
+	 * something like half a pixel left over.
+	 */
+	if (!needcol && (dest_column < new_width)) {
+	    nrows[destrow][dest_column * BPP] = r;
+	    nrows[destrow][1+dest_column * BPP] = g;
+	    nrows[destrow][2+dest_column * BPP] = b;
+	    nrows[destrow][3+dest_column * BPP] = a;
 	}
 	destrow++;
     }
@@ -538,12 +543,12 @@ int init_pngx_loader(Display *display)
     ximage = XCreateImage(display, visual,
 		      depth,
 		      ZPixmap, 0, 0, 
-		      32, 32,  pad, 0);
+		      MAX_IMAGE_SIZE, MAX_IMAGE_SIZE,  pad, 0);
     if (!ximage) {
 	fprintf(stderr,"Failed to create Ximage\n");
 	return 1;
     }
-    ximage->data = malloc(ximage->bytes_per_line * 32);
+    ximage->data = malloc(ximage->bytes_per_line * MAX_IMAGE_SIZE);
     if (!ximage->data) {
 	fprintf(stderr,"Failed to create Ximage data\n");
 	return 1;
@@ -780,7 +785,6 @@ int rgba_to_xpixmap(Display *display, Drawable draw, uint8 *pixels,
 	cmask, lastcmask, lastcolor;
     GC	gc, gc_alpha;
 
-
     *pix = XCreatePixmap(display, draw, width, height, 
 			DefaultDepth(display,  DefaultScreen(display)));
 
@@ -838,7 +842,7 @@ int rgba_to_xpixmap(Display *display, Drawable draw, uint8 *pixels,
 	}
     }
 
-    XPutImage(display, *pix, gc, ximage, 0, 0, 0, 0, 32, 32);
+    XPutImage(display, *pix, gc, ximage, 0, 0, 0, 0, width, height);
     XFreeGC(display, gc_alpha);
     XFreeGC(display, gc);
     return 0;
@@ -861,6 +865,9 @@ int create_and_rescale_image_from_data(Cache_Entry *ce, int pixmap_num, uint8 *r
 		   &pixmaps[pixmap_num]->mask, &colormap, width, height);
 
     if (!pixmaps[pixmap_num]->pixmap || !pixmaps[pixmap_num]->mask) return 1;
+    pixmaps[pixmap_num]->width = width / image_size;
+    pixmaps[pixmap_num]->height = height / image_size;
+
     if (ce) {
 	ce->image_data = pi;
     }
@@ -874,8 +881,8 @@ void get_map_image_size(int face, uint8 *w, uint8 *h)
 	*w = 1;
 	*h = 1;
     }
-    *w = 1;
-    *h = 1;
+    *w = pixmaps[face]->width;
+    *h = pixmaps[face]->height;
 }
 
 
